#!/bin/bash

#
# Script
#
# g4cParallelSimulation.sh
#
#   (1) Imposta i parametri della simulazione parallela
#   (2) Controlla la correttezza dei parametri della simulazione
#   (3) Lancia il setup dell'ambiente necessario alla esecuzione parallela
#       dei job che realizzano la simulazione e ne controlla l'esito
#   (4) Lancia l'esecuzione parallela dei job che realizzano la simulazione
#       e ne controlla l'esito
#   (5) Lancia la raccolta dei dettagli relativi ai tempi di esecuzione
#       dei singoli job in un unico file e ne controlla l'esito
#

#
# Imposta i parametri della simulazione parallela
#
# <verbose>
#   i valori ammessi per il parametro sono 0 o 1.
#   Se 1 attiva la modalita' verbose di esecuzione dello script
# <macroFileAbsolutePath>
#   path assoluto del file di macro a partire dal quale vengono generati
#   quelli per le simulazioni eseguite dai singoli job
# <workingDirectoryRelativePath>
#   path relativo alla directory g4cParallelSimulation di quella
#   di lavoro che contiene le subdirectory assegnate ai job che
#   realizzano la simulazione, il file con le informazioni relative
#   ai loro istanti di sottomissione e di completamento e il file di
#   output per la modalita' verbose
# <totalNumberOfJobs>
#   numero totale dei job che realizzano la simulazione parallela.
#   Deve preferibilmente essere minore o uguale del numero di host
#   impegnati moltiplicato per il minimo numero di core riservati alla
#   simulazione su ogni host
# <simulationExecutableAbsolutePath>
#   path assoluto dell'eseguibile da assegnare ad ogni job
#   della simulazione parallela
# <jobsTimeOut>
#   numero di secondi dall'inizio della sua esecuzione trascorso il quale
#   ogni job viene interrotto. Se 0 inibisce il time out
# <parallelJobsPerHost>
#   numero dei job eseguiti in parallelo su ognuno degli host impegnati nella
#   simulazione. Non deve essere maggiore del minimo numero di core riservati
#   alla simulazione su ogni host
# <hostsList>
#   elenco dei nomi degli host impegnati nella simulazione, separati da
#   virgola e senza blank
# <beamCmdString>
#   stringa che nel file di macro individua la direttiva che setta 
#   il numero degli eventi processati da ogni job
# <setSeedsString>
#   stringa che nel file di macro individua la direttiva che setta
#   il seed di generazione dei numeri casuali
# <verbOutputFileName>
#   nome del file di output per la modalita' verbose
# <jobsRuntimeFileName>
#   nome del file, nella working directory, nel quale vengono
#   memorizzati gli istanti di sottomissione e di completamento
#   di tutti i job che realizzano la simulazione 
# <jobRuntimeFileName>
#   nome del file, nella directory assegnata ad ogni job, nel
#   quale vengono memorizzati i suoi istanti di sottomissione e
#   di completamento 
# <geant4EnvVariablesSetUp>
#   path assoluto dello script che setta le variabili di ambiente
#   di Geant4 per la simulazione
# <tspQueuesPositioning>
#   variabile simbolica che, definendo il posizionamento del file
#   di informazioni per la coda di task gestita da tsp 
#   (task spooler), ne permette l'accesso all'host che la
#   utilizza per gestire l'esecuzione parallela dei job.
#   Al file in questione, memorizzato su ogni host nella directory
#   "\tmp",  assegnato il nome della working directory seguito
#   dal suffisso ".tmp"
# <scriptsRelativePath>
#   path relativo alla directory g4cParallelSimulation
#   di quella che contiene gli script principali
#   che implementano la simulazione
# <errRelativePath>
#   path relativo alla directory g4cParallelSimulation
#   di quella che contiene gli script per i messaggi
#   di errore
# <verbRelativePath>
#   path relativo alla directory g4cParallelSimulation 
#   di quella che contiene gli script di gestione 
#   della modalita' verbose
# <helpRelativePath>
#   path relativo alla directory g4cParallelSimulation
#   di quella che contiene i messaggi di aiuto al lancio
#   della simulazione
#
source ./g4cParallelSimulationParametersSetup.sh

# Accede al nome dello script correntemente in esecuzione
currentScript=$(basename $0)

# Inizializza a "false" (0) la variabile che tiene traccia della
# errata specifica di almeno un parametro
wrongParameters=0										  
							
# Controlla se il path assoluto del file di macro e' correttamene
# specificato.
if test ! -f $macroFileAbsolutePath
then
   # In caso di errore
   # Comunica la circostanza
   $errRelativePath/errMacroFileAbsolutePath.sh $currentScript \
                                                $macroFileAbsolutePath

   # Setta a "true" (1) la variabile che tiene traccia della errata
   # specifica di almeno un parametro
   wrongParameters=1
fi

# Controlla se il path assoluto del file eseguibile e' correttamene
# specificato. 
if test ! -f $simulationExecutableAbsolutePath
then
   # In caso di errore

   # Comunica la circostanza
   $errRelativePath/errExecFileAbsolutePath.sh $currentScript \
                                               $simulationExecutableAbsolutePath
   
   # Setta a "true" (1) la variabile che tiene traccia della errata
   # specifica di almeno un parametro
   wrongParameters=1
fi

# Controlla se il parametro <verbose> e' correttamente specificato
# (valori ammessi 1 o 0). 
if test $verbose -ne 0 && test $verbose -ne 1
then
   # In caso di errore

   # Comunica la circostanza
   $errRelativePath/errVerboseValue.sh $currentScript

   # Setta a "true" (1) la variabile che tiene traccia della errata
   # specifica di almeno un parametro
   wrongParameters=1
fi

# Estrae, se presente, dal file di macro la direttiva che specifica il
# numero degli eventi processati da ogni job
beamString=$(grep "$beamCmdString" "$macroFileAbsolutePath" 2>&1)

# Controlla la presenza nel file di macro della direttiva che specifica
# il numero degli eventi processati da ogni job
if test -z "$beamString"
then
   # In caso di errore

   # Comunica la circostanza
   $errRelativePath/errNumberOfEvents.sh $currentScript
   
   # Setta a "true" (1) la variabile che tiene traccia della errata
   # specifica di almeno un parametro
   wrongParameters=1
fi

# Estrae, se presente, dal file di macro la direttiva che specifica il
# seed di generazione casuale di eventi
randString=$(grep "$setSeedsString" $macroFileAbsolutePath 2>&1)

# Controlla la presenza nel file di macro della direttiva che specifica
# il seed di generazione casuale di eventi
if test -z "$randString"
then
   # In caso di errore

   # Comunica la circostanza
   $errRelativePath/errEventsRandomGenerationSeed.sh $currentScript
									 
   # Setta a "true" (1) la variabile che tiene traccia della errata
   # specifica di almeno un parametro
   wrongParameters=1
fi

# Se qualcuno dei parametri  stato specificato
# in maniera non corretta, interrompe la simulazione
if test $wrongParameters -ne 0 
then
   # Segnala l'avvenuta interruzione della simulazione
   $helpRelativePath/helpAbortedSimulation.sh
   
   exit 1
fi

# Se la working directory non esiste, la crea.
# Altrimenti, previa conferma, ne azzera il contenuto;
# in caso di mancata conferma, interrompe la simulazione

# Controlla l'esistenza della working directory
if test -d $workingDirectoryRelativePath
then
   msg=$($helpRelativePath/helpOverwritingWorkingDirectory.sh)

   while true; do
      # Nel caso in cui la working directory esista,
      # chiede conferma della volonta' di azzerarne 
      # il contenuto
      read -p "$msg" answer

      case $answer in
        [Yy]* ) \
                                                             
        # In caso di risposta affermativa ne azzera il contenuto                                                     
        rm -rf $workingDirectoryRelativePath/*;
		
        break;;				
        [Nn]* ) \
		# in caso di risposta negativa interrompe la simulazione
		# Help 
		# Segnala l'avvenuta interruzione della simulazione
		$helpRelativePath/helpAbortedSimulation.sh;
        
		exit 0;;
        * ) echo "Rispondere Yes o No";;
      esac
   done
else
   # Se la working directory non esiste la crea
   mkdir -p $workingDirectoryRelativePath
fi		  

# Verbose
#   Inizializza il file di output per la modalita' verbose, se attiva
$scriptsRelativePath/g4cVerbOutputFileInit.sh $verbose \
        $workingDirectoryRelativePath/$verbOutputFileName
                                             
# Verbose
#   Memorizza nel file di output per la modalita' verbose, se attiva, 
#   l'inizio della esecuzione dello script
$verbRelativePath/verbScriptStarting.sh $verbose \
        $currentScript \
        $workingDirectoryRelativePath/$verbOutputFileName
  										 
# Verbose
#   Memorizza nel file di output per la modalita' verbose, se attiva,
#   il valore dei parametri della simulazione parallela
$verbRelativePath/verbParallelSimulationParameters.sh $verbose \
                 $currentScript \
                 $macroFileAbsolutePath \
                 $workingDirectoryRelativePath \
                 $totalNumberOfJobs \
                 $simulationExecutableAbsolutePath \
                 $jobsTimeOut \
                 $parallelJobsPerHost \
                 $hostsList \
                 $geant4EnvVariablesSetUp \
				 $tspQueuesPositioning \
                 $beamCmdString \
                 $setSeedsString \
                 $verbOutputFileName \
                 $jobsRuntimeFileName \
                 $jobRuntimeFileName \
                 $scriptsRelativePath \
                 $errRelativePath \
                 $verbRelativePath \
                 $helpRelativePath \
                 $workingDirectoryRelativePath/$verbOutputFileName

# Predispone l'ambiente per l'esecuzione parallela dei job
# che realizzano la simulazione
$scriptsRelativePath/g4cParallelEnvSetup.sh $verbose \
                    $macroFileAbsolutePath \
                    $setSeedsString \
                    $workingDirectoryRelativePath \
                    $totalNumberOfJobs \
                    $verbRelativePath \
                    $workingDirectoryRelativePath/$verbOutputFileName
																				
# Se la predisposizione dell'ambiente per l'esecuzione parallela dei job fallisce
# segnala l'interruzione anomala della simulazione e termina
if test $? -ne 0 
then
   # Error
   $errRelativePath/errParallelEnvSetup.sh $currentScript
  												   
   #  Verbose
   #    Memorizza nel file di output per la modalita' verbose
   #    il fallimento del setup dell'ambiente necessario
   #    alla esecuzione parallela dei job che realizzano
   #    la simulazione
   $verbRelativePath/verbParallelEnvSetupFailure.sh $verbose \
                    $currentScript \
                    $workingDirectoryRelativePath/$verbOutputFileName
   													   
   #  Verbose
   #    memorizza nel file di output per la modalita' verbose, se attiva,
   #   il completamento dello script
   $verbRelativePath/verbScriptCompletion.sh $verbose \
                    $currentScript \
                    $workingDirectoryRelativePath/$verbOutputFileName
   
exit 1
fi

# Lancia l'esecuzione parallela dei job che realizzano la simulazione
$scriptsRelativePath/g4cParallelJobsExecution.sh $verbose \
                    $macroFileAbsolutePath \
                    $workingDirectoryRelativePath \
                    $simulationExecutableAbsolutePath \
                    $jobsTimeOut \
                    $parallelJobsPerHost \
                    $hostsList \
                    $jobRuntimeFileName \
                    $geant4EnvVariablesSetUp \
					$tspQueuesPositioning \
                    $helpRelativePath \
                    $verbRelativePath \
                    $workingDirectoryRelativePath/$verbOutputFileName
                                               
# Se l'esecuzione parallela dei job fallisce segnala l'interruzione anomala
# della simulazione e termina
if test $? -ne 0
then
   # Error
   $errRelativePath/errJobsExecution.sh $currentScript
  													   
   # Verbose
   $verbRelativePath/verbJobsExecution.sh $verbose \
                                          $currentScript \
                                          $verbOutputFileRelativePath
   												   
   #  Verbose
   #    Memorizza nel file di output per la modalita' verbose, se attiva, 
   #    l'uscita dallo script
   $verbRelativePath/verbScriptCompletion.sh $verbose \
                                             $currentScript \
                                             $verbOutputFileRelativePath 
exit 1
fi

# Raccoglie in un unico file gli istanti di
# sottomissione e di completamento dei singoli job
$scriptsRelativePath/g4cJobsExecTimesCollection.sh $verbose \
                    $workingDirectoryRelativePath \
                    $jobRuntimeFileName \
                    $jobsRuntimeFileName \
                    $verbRelativePath \
                    $workingDirectoryRelativePath/$verbOutputFileName
												   
# Se la raccolta in un unico file degli istanti di
# sottomissione e completamento dei singoli job
# fallisce segnala l'interruzione anomala
# della simulazione e termina
if test $? -ne 0
then
   # Error
   $errRelativePath/errJobsExecutionTimes.sh $currentScript
  													   
   # Verbose
   $verbRelativePath/verbJobsExecutionTimes.sh $verbose \
                                               $currentScript \
                                               $verbOutputFileRelativePath
   												   
   #  Verbose
   #    Memorizza nel file di output per la modalit verbose, se attiva, 
   #    l'uscita dallo script
   $verbRelativePath/verbScriptCompletion.sh $verbose \
                                             $currentScript \
                                             $verbOutputFileRelativePath
    
exit 1
fi

#  Verbose
#    Memorizza nel file di output per la modalit verbose, se attiva, 
#    l'uscita dallo script
$verbRelativePath/verbScriptCompletion.sh $verbose \
                                          $currentScript \
                                          $workingDirectoryRelativePath/$verbOutputFileName			  
exit 0

